import { ConditionId } from './template';
import * as Functions from './functions';
export declare type TODO = any;
export declare type Diff<T extends string, U extends string> = ({
    [P in T]: P;
} & {
    [P in U]: never;
} & {
    [x: string]: never;
})[T];
export declare type Omit<T, K extends keyof T> = Pick<T, Diff<keyof T, K>>;
export declare type CFString = string;
export declare type CFBoolean = 'true' | 'false';
export declare function CFInteger(n: number): string;
export declare type CFInteger = string;
export declare function CFDouble(n: number): string;
export declare type CFDouble = string;
export declare function CFLong(n: number): string;
export declare type CFLong = string;
export declare function CFTimestamp(d: string): string;
export declare type CFTimestamp = string;
export declare type CFJson = object;
export declare type CFList<T> = Array<T>;
export declare type CFMap<T> = {
    [key: string]: T;
};
/** For situations in which the *only* function you may use is a reference */
export declare type LiteralOrRef<V = any> = V | Ref;
export declare type BooleanCondition = Functions.And | Functions.Equals | Functions.Not | Functions.Or | ConditionRef;
/** Reference to a conditional declared elsewhere in the stack template */
export interface ConditionRef {
    Condition: ConditionId;
}
/**
 * Yields a value of type V.  In other words, can appear in the JSON where that type of value is expected and where functions are allowed to execute.
 * Can be a function that returns the value, or can be the literal value itself.
 */
export interface Value<V> {
    __brandCloudfrontValue: V;
}
export declare type Yields<V> = Value<V>;
export declare type YieldsString = Yields<CFString>;
export declare type YieldsBoolean = Yields<CFBoolean>;
declare global  {
    interface String extends Value<string> {
    }
    interface Number extends Value<number> {
    }
    interface Array<T> extends Value<Array<T>> {
    }
}
/**
 * This is some sort of function or reference that ultimately yields a value of type V.
 * Allows the type system to understand what type a condition, value, reference, etc. will return.
 */
export interface AbstractResource {
    type: string;
    Condition?: ConditionId;
    DeletionPolicy?: 'Delete' | 'Retain' | 'Snapshot';
    DependsOn?: CFString | Array<CFString>;
    Metadata?: object;
}
export interface Ref {
    Ref: CFString;
}
export declare function Ref<V>(ref: Ref['Ref']): Ref & Value<V>;
